'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var isAccepted = _interopDefault(require('attr-accept'));
var React = require('react');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __spreadArrays() {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
}

var MAX_SIZE_ERROR = "max_size_error";
var FILE_TYPE_ERROR = "file_type_error";

var isDragEvent = function (event) { return event.hasOwnProperty("dataTransfer"); };
var getFilesFromEvent = function (event) {
    return isDragEvent(event) ? event.dataTransfer.files : event.target.files;
};
var checkFileSize = function (file, maxSize) {
    if (!maxSize || file.size <= maxSize) {
        return;
    }
    return MAX_SIZE_ERROR;
};
var checkFileType = function (file, accept) {
    if (!accept || isAccepted(file, accept)) {
        return;
    }
    return FILE_TYPE_ERROR;
};
var validateFile = function (_a) {
    var file = _a.file, maxSize = _a.maxSize, accept = _a.accept;
    var maxSizeError = checkFileSize(file, maxSize);
    var fileTypeError = checkFileType(file, accept);
    return [maxSizeError, fileTypeError].filter(Boolean);
};

/**
 * Provides access:
 *
 * - to attached files by DnD or with help of input[type="file"];
 * - to validation errors;
 * - to the api to manage the attached file.
 *
 * @param {Object} [config] Configuration options object.
 * @param {number} [config.maxSize] Maximum bytes of data allowed to upload.
 * @param {string} [config.accept] Files types allowed to upload.
 * @param {boolean} [config.multiple=false] To allow or not multiple files upload.
 * @param {onAttach} [config.onAttach=() => {}] To allow or not multiple files upload.
 *
 * @returns {Object} api Api to manage attached files.
 */
var urlsToAcceptedFilesData = function (urls) {
    return urls.map(function (url) { return ({ url: url }); });
};
var useUploaderHook = function (_a) {
    var maxSize = _a.maxSize, accept = _a.accept, _b = _a.multiple, multiple = _b === void 0 ? false : _b, _c = _a.onAttach, onAttachCb = _c === void 0 ? function () { } : _c, _d = _a.uploadedFilesUrls, uploadedFilesUrls = _d === void 0 ? [] : _d;
    var _e = React.useState(function () { return urlsToAcceptedFilesData(uploadedFilesUrls); }), acceptedFilesData = _e[0], setAcceptedFilesData = _e[1];
    var _f = React.useState([]), rejectedFilesData = _f[0], setRejectedFilesData = _f[1];
    var _g = React.useState([]), removedFilesUrls = _g[0], setRemovedFilesUrls = _g[1];
    var rootRef = React.useRef();
    var inputRef = React.useRef();
    var handleAttach = function (event) {
        event.stopPropagation();
        event.preventDefault();
        var files = Array.from(getFilesFromEvent(event));
        if (!multiple) {
            files = [files[0]];
        }
        var _a = files.reduce(function (result, file) {
            var errors = validateFile({ file: file, accept: accept, maxSize: maxSize });
            if (!errors.length) {
                result.accepted.push({ file: file, preview: URL.createObjectURL(file) });
            }
            else {
                result.rejected.push({ file: file, errors: errors });
            }
            return result;
        }, { accepted: [], rejected: [] }), accepted = _a.accepted, rejected = _a.rejected;
        updateFilesData(accepted);
        updateFilesData(rejected);
        onAttachCb(accepted, rejected, event);
    };
    var getInputProps = function () { return ({
        name: "uploader",
        ref: inputRef,
        onChange: handleAttach,
        multiple: multiple,
        accept: accept
    }); };
    var getRootProps = function () { return ({
        ref: rootRef,
        onDrop: handleAttach
    }); };
    var isAcceptedData = function (data) {
        return !data[0].hasOwnProperty("errors");
    };
    var updateFilesData = function (data) {
        if (!data.length) {
            return;
        }
        if (isAcceptedData(data)) {
            var resultData = multiple ? __spreadArrays(acceptedFilesData, data) : data;
            setAcceptedFilesData(resultData);
        }
        else {
            var resultData = multiple ? __spreadArrays(rejectedFilesData, data) : data;
            setRejectedFilesData(resultData);
        }
    };
    var deleteFile = function (indexToDelete) {
        var removedFile = acceptedFilesData[indexToDelete];
        var resultAcceptedFilesData = acceptedFilesData.filter(function (_, index) { return indexToDelete !== index; });
        setAcceptedFilesData(resultAcceptedFilesData);
        if ("url" in removedFile) {
            setRemovedFilesUrls(__spreadArrays(removedFilesUrls, [removedFile]));
        }
    };
    var deleteAllFiles = function () {
        setAcceptedFilesData([]);
        setRejectedFilesData([]);
        var removedUploadedFiles = acceptedFilesData.filter(function (data) { return "url" in data; });
        setRemovedFilesUrls(__spreadArrays(removedFilesUrls, removedUploadedFiles));
    };
    return {
        getInputProps: getInputProps,
        getRootProps: getRootProps,
        deleteFile: deleteFile,
        deleteAllFiles: deleteAllFiles,
        acceptedFilesData: acceptedFilesData,
        rejectedFilesData: rejectedFilesData,
        removedFilesUrls: removedFilesUrls
    };
};

var DropZone = function (_a) {
    var children = _a.children, props = __rest(_a, ["children"]);
    var result = useUploaderHook(props);
    return React.createElement(React.Fragment, null, children(result));
};

exports.useUploaderHook = useUploaderHook;
exports.DropZone = DropZone;
exports.MAX_SIZE_ERROR = MAX_SIZE_ERROR;
exports.FILE_TYPE_ERROR = FILE_TYPE_ERROR;
